/*
 * QAuxiliary - An Xposed module for QQ/TIM
 * Copyright (C) 2019-2022 qwq233@qwq2333.top
 * https://github.com/cinit/QAuxiliary
 *
 * This software is non-free but opensource software: you can redistribute it
 * and/or modify it under the terms of the GNU Affero General Public License
 * as published by the Free Software Foundation; either
 * version 3 of the License, or any later version and our eula as published
 * by QAuxiliary contributors.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * and eula along with this software.  If not, see
 * <https://www.gnu.org/licenses/>
 * <https://github.com/cinit/QAuxiliary/blob/master/LICENSE.md>.
 */

package io.github.qauxv.util.dexkit

import io.github.qauxv.config.ConfigManager
import io.github.qauxv.util.dexkit.impl.DexKitDeobfs
import io.github.qauxv.util.dexkit.impl.LegacyDexDeobfs
import java.util.concurrent.atomic.AtomicInteger

object DexDeobfsProvider {

    private const val KEY_DEX_DEOBFS_BACKEND = "dex_deobfs_backend2"

    private val mDeobfsSection = AtomicInteger(0)

    fun enterDeobfsSection() {
        mDeobfsSection.incrementAndGet()
    }

    fun exitDeobfsSection() {
        mDeobfsSection.decrementAndGet()
    }

    /**
     * Create a new instance. Call [DexDeobfsBackend.close] when you are done.
     */
    fun getCurrentBackend(): DexDeobfsBackend {
        check(mDeobfsSection.get() > 0) { "dex deobfuscation is not meant to be available now" }
        val id = ConfigManager.getDefaultConfig()
            .getString(KEY_DEX_DEOBFS_BACKEND, DexKitDeobfs.ID)
        return when (id) {
            LegacyDexDeobfs.ID -> LegacyDexDeobfs.newInstance()
            DexKitDeobfs.ID -> DexKitDeobfs.newInstance()
            else -> throw IllegalArgumentException("Unknown dex deobfs backend: $id")
        }
    }

    var currentBackendId: String
        get() = ConfigManager.getDefaultConfig().getString(KEY_DEX_DEOBFS_BACKEND, DexKitDeobfs.ID)!!
        set(value) {
            require(value in listOf(LegacyDexDeobfs.ID, DexKitDeobfs.ID)) {
                "Unknown dex deobfs backend: $value"
            }
            ConfigManager.getDefaultConfig().putString(KEY_DEX_DEOBFS_BACKEND, value)
        }

    val currentBackendName: String
        get() = when (currentBackendId) {
            LegacyDexDeobfs.ID -> LegacyDexDeobfs.NAME
            DexKitDeobfs.ID -> DexKitDeobfs.NAME
            else -> throw IllegalArgumentException("Unknown dex deobfs backend: $currentBackendId")
        }

    val allBackendNames: List<Pair<String, String>> = listOf(
        LegacyDexDeobfs.ID to LegacyDexDeobfs.NAME,
        DexKitDeobfs.ID to DexKitDeobfs.NAME,
    )

    val isDexKitBackend get() = currentBackendId == DexKitDeobfs.ID
}
